@@ -116,16 +116,17 @@ VP.gllvm <- function(object, group = NULL, groupnames=NULL, adj.cov = TRUE, grou
       object$col.eff$Xt <- object$X.design
     }
     bars <- findbars1(object$col.eff$col.eff.formula) # list with 3 terms
-    groupnamesF <- paste0("Random effect: ", unlist(lapply(bars,safeDeparse)))
+    safeDeparse <- function(x) paste(deparse(x, 500L), collapse = " ")
+    
+    groupnamesF <- c(groupnamesF, paste0("Random effect: ", unlist(lapply(bars,safeDeparse))))
     
     mf <- model.frame(subbars1(reformulate(sprintf("(%s)", sapply(findbars1(object$col.eff$col.eff.formula), deparse1)))),data=data.frame(object$col.eff$Xt))
 
-    safeDeparse <- function(x) paste(deparse(x, 500L), collapse = " ")
     names(bars) <- vapply(bars, function(x) paste(deparse(x[[3]], 500L), collapse = " "), "")
 
     blist <- lapply(bars, mkModMlist, mf)
     groupFs <- table(lapply(blist, function(x)row.names(x$sm)%in%colnames(object$col.eff$spdr)))
-    groupF <- rep(1:length(groupFs), groupFs)
+    groupF <- c(groupF, rep(1:length(groupFs), groupFs)+max(groupF))
     
     X.d <- object$col.eff$spdr#[, colnames(object$col.eff$spdr)!= "Intercept", drop=FALSE]
     x_in_model = colnames(X.d)
@@ -182,6 +183,7 @@ VP.gllvm <- function(object, group = NULL, groupnames=NULL, adj.cov = TRUE, grou
         CoefMat <- rbind(CoefMat, Bt)
         LVgroups = c(LVgroups, 1:(object$num.lv.c + object$num.RR))
       }
+      if(object$num.lv>0)groupnamesF <- c(groupnamesF, paste0("LV", 1:object$num.lv))
       
       # lvs/unconstr. part
       if (object$num.RR == 0) {
@@ -200,6 +202,7 @@ VP.gllvm <- function(object, group = NULL, groupnames=NULL, adj.cov = TRUE, grou
         } else if (object$num.lv > 0 & object$num.lv.c == 0) {
             lvs <- cbind(matrix(0, ncol = object$num.RR, 
                         nrow = n), t(t(object$lvs) * object$params$sigma.lv))
+        groupnamesF <- c(groupnamesF, paste0("LV", 1:object$num.lv))
         } else {
             lvs <- matrix(0, ncol = object$num.RR, nrow = n)
         }
@@ -316,7 +319,13 @@ VP.gllvm <- function(object, group = NULL, groupnames=NULL, adj.cov = TRUE, grou
   }
   
   if(is.null(groupnames)) {
-    if(!is.null(groupnamesF)){
+    if(!is.null(groupnamesF) && is.null(groupnames)){
+      groupnames <- vector("character", max(group))
+      
+        for (k in unique(group)) {
+          groupnames[k] <- paste(rownames(CoefMat)[group==k],collapse="/")
+        }
+    }else if(is.null(groupnames)){
       groupnames <- groupnamesF
     }
     
