@@ -165,6 +165,7 @@ residuals.gllvm <- function(object, ...) {
         ds.res <- matrix(qnorm(u),n,p)
       }
       if (object$family == "ZIP") {
+        mu = exp(eta.mat)
         b <- pzip(as.vector(y), mu = as.vector(mu), sigma = rep(object$params$phi, each = n))
         a <- pmin(b, pzip(as.vector(y) - 1, mu = as.vector(mu), sigma = rep(object$params$phi, each = n)))
         
@@ -175,6 +176,7 @@ residuals.gllvm <- function(object, ...) {
         ds.res <- matrix(qnorm(u),n,p)
       }
       if (object$family == "ZINB") {
+        mu = exp(eta.mat)
         b <- pzinb(as.vector(y), mu = as.vector(mu), p = rep(object$params$phi, each = n), sigma = rep(object$params$ZINB.phi, each = n))
         a <- pmin(b, pzinb(as.vector(y) - 1, mu = as.vector(mu), p = rep(object$params$phi, each = n), sigma = rep(object$params$ZINB.phi, each = n)))
 
@@ -197,6 +199,35 @@ residuals.gllvm <- function(object, ...) {
         if(any(u==0, na.rm = TRUE)&&replace)u[u==0] <- 1e-16
         ds.res <- matrix(qnorm(u),n,p)
       }
+      if (object$family == "ZIB") {
+        if(length(Ntrials)==1)Ntrials <- rep(Ntrials,p)
+        if(length(Ntrials)==p)Ntrials <- rep(Ntrials, each = n)
+        
+        b <- pzib(as.vector(y), Ntrials = Ntrials, mu = as.vector(mu), sigma = rep(object$params$phi, each = n))
+        a <- pmin(b, pzib(as.vector(y) - 1, Ntrials = Ntrials, mu = as.vector(mu), sigma = rep(object$params$phi, each = n)))
+        
+        u = a+(b-a)*runif(n*p)
+        
+        if(any(u==1, na.rm = TRUE)&&replace)u[u==1] <- 1-1e-16
+        if(any(u==0, na.rm = TRUE)&&replace)u[u==0] <- 1e-16
+        ds.res <- matrix(qnorm(u),n,p)
+      }
+      if (object$family == "ZNIB") {
+        if(length(Ntrials)==1)Ntrials <- rep(Ntrials,p)
+        if(length(Ntrials)==p)Ntrials <- rep(Ntrials, each = n)
+        
+        phis0 = object$params$phi/(1+object$params$phi + object$params$ZINB.phi);
+        phisN = object$params$ZINB.phi/(1+object$params$phi + object$params$ZINB.phi);
+        
+        b <- pznib(as.vector(y), Ntrials = Ntrials, mu = as.vector(mu), p0 = rep(phis0, each = n), pN = rep(phisN, each = n))
+        a <- pmin(b, pznib(as.vector(y) - 1, Ntrials = Ntrials, mu = as.vector(mu), p0 = rep(phis0, each = n), pN = rep(phisN, each = n)))
+        
+        u = a+(b-a)*runif(n*p)
+        
+        if(any(u==1, na.rm = TRUE)&&replace)u[u==1] <- 1-1e-16
+        if(any(u==0, na.rm = TRUE)&&replace)u[u==0] <- 1e-16
+        ds.res <- matrix(qnorm(u),n,p)
+      }
       if (object$family == "tweedie") {
         phis <- object$params$phi + 1e-05
         b <- fishMod::pTweedie(as.vector(y), mu = as.vector(mu), phi = rep(phis, each = n), p = object$Power)
@@ -211,16 +242,17 @@ residuals.gllvm <- function(object, ...) {
         ds.res <- matrix(qnorm(u),n,p)
       }
       if (object$family == "ordinal") {
+        linkfun <- switch(object$link, "probit" = pnorm, "logit" = plogis)
         for (i in 1:n) {
           for (j in 1:p) {
           if(object$zeta.struc == "species"){
             probK <- NULL
-            probK[1] <- pnorm(object$params$zeta[j, 1] - eta.mat[i, j], log.p = FALSE)
-            probK[max(y[, j]) + 1 - min(y[, j])] <- 1 - pnorm(object$params$zeta[j, max(y[, j]) - min(y[, j])] - eta.mat[i, j])
+            probK[1] <- linkfun(object$params$zeta[j, 1] - eta.mat[i, j], log.p = FALSE)
+            probK[max(y[, j]) + 1 - min(y[, j])] <- 1 - linkfun(object$params$zeta[j, max(y[, j]) - min(y[, j])] - eta.mat[i, j])
             if(length(unique(y[,j]))>2) {
               j.levels <- 2:(max(y[, j]) - min(y[, j]))#
               for (k in j.levels) {
-                probK[k] <- pnorm(object$params$zeta[j, k] - eta.mat[i, j]) - pnorm(object$params$zeta[j, k - 1] - eta.mat[i, j])
+                probK[k] <- linkfun(object$params$zeta[j, k] - eta.mat[i, j]) - linkfun(object$params$zeta[j, k - 1] - eta.mat[i, j])
               }
             }
             probK <- c(0, probK)
@@ -234,11 +266,11 @@ residuals.gllvm <- function(object, ...) {
             ds.res[i, j] <- qnorm(u)
           } else {
             probK <- NULL
-            probK[1] <- pnorm(object$params$zeta[1] - eta.mat[i, j], log.p = FALSE)
-            probK[max(y) + 1 - min(y)] <- 1 - pnorm(object$params$zeta[max(y) - min(y)] - eta.mat[i, j])
+            probK[1] <- linkfun(object$params$zeta[1] - eta.mat[i, j], log.p = FALSE)
+            probK[max(y) + 1 - min(y)] <- 1 - linkfun(object$params$zeta[max(y) - min(y)] - eta.mat[i, j])
               levels <- 2:(max(y) - min(y))#
               for (k in levels) {
-                probK[k] <- pnorm(object$params$zeta[k] - eta.mat[i, j]) - pnorm(object$params$zeta[k - 1] - eta.mat[i, j])
+                probK[k] <- linkfun(object$params$zeta[k] - eta.mat[i, j]) - linkfun(object$params$zeta[k - 1] - eta.mat[i, j])
               }
             probK <- c(0, probK)
             cumsum.b <- sum(probK[1:(y[i,j]+ifelse(min(y)==0,1,0) + 1)])
