@@ -106,7 +106,7 @@ se.gllvm <- function(object, ...){
       # if(object$beta0com){ incl[names(objrFinal$par)=="b"] <- FALSE}
       if(familyn!=7 & familyn!=12) incl[names(objrFinal$par)=="zeta"] <- FALSE
       if(familyn==0 || familyn==2 || familyn==7 || familyn==8) incl[names(objrFinal$par)=="lg_phi"] <- FALSE
-      if(familyn!=11) incl[names(objrFinal$par)=="lg_phiZINB"] <- FALSE
+      if(!familyn %in% c(11, 14)) incl[names(objrFinal$par)=="lg_phiZINB"] <- FALSE
       
       if(num.lv>0) {
         incld[names(objrFinal$par)=="Au"] <- TRUE
@@ -230,6 +230,7 @@ se.gllvm <- function(object, ...){
       
       if(num.lv > 0) {
         se.sigma.lv <- se$sigmaLV[1:num.lv]
+        # se.sigma.lv <- se.sigma.lv*object$params$sigma.lv[1:num.lv]
         se.lambdas <- matrix(0,p,num.lv); se.lambdas[lower.tri(se.lambdas, diag=FALSE)] <- se$lambda[1:(p * num.lv - sum(0:num.lv))];
         colnames(se.lambdas) <- paste("LV", 1:num.lv, sep="");
         rownames(se.lambdas) <- colnames(object$y)
@@ -264,15 +265,16 @@ se.gllvm <- function(object, ...){
       names(out$sd$B) <- names(object$params$B)
       if(!is.null(object$params$row.params.fixed)) {out$sd$row.params.fixed <- se.row.params}
       
-      if(family %in% c("ZINB")) {
-        se.ZINB.lphis <- se$lg_phiZINB[disp.group];  out$sd$ZINB.inv.phi <- se.ZINB.lphis*object$params$ZINB.inv.phi;
+      if(family %in% c("ZINB", "ZNIB")) {
+        se.ZINB.lphis <- se$lg_phiZINB[disp.group];  
+        if(family == "ZINB")out$sd$ZINB.inv.phi <- se.ZINB.lphis*object$params$ZINB.inv.phi;
         out$sd$ZINB.phi <- se.ZINB.lphis*object$params$ZINB.phi;
         names(out$sd$ZINB.phi) <- colnames(object$y);
         
         if(!is.null(names(disp.group))){
           try(names(out$sd$ZINB.phi) <- names(disp.group),silent=T)
         }
-        names(out$sd$ZINB.inv.phi) <-  names(out$sd$ZINB.phi)
+        if(family == "ZINB")names(out$sd$ZINB.inv.phi) <-  names(out$sd$ZINB.phi)
       }
       
       if(family %in% c("negative.binomial")) {
@@ -300,15 +302,27 @@ se.gllvm <- function(object, ...){
         }
       }
       
-      if(family%in%c("ZIP","ZINB")) {
+      if(family%in%c("ZIP","ZINB","ZIB", "ZNIB")) {
         pars <- object$TMBfn$par
         p0i <- names(pars)=="lg_phi"
         p0 <- pars[p0i]
       }
       
-      if(family %in% c("ZIP","ZINB")) {
+      if(family %in% c("ZIP","ZINB","ZIB")) {
         se.phis <- se$lg_phi[disp.group];
         out$sd$phi <- se.phis*exp(p0)/(1+exp(p0))^2;#
+        if(length(unique(disp.group))==p){
+          names(out$sd$phi) <- colnames(object$y);
+        }else if(!is.null(names(disp.group))){
+          try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+        }else{
+          names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+        }
+      }else if(family == "ZNIB"){
+        se.phis <- se$lg_phi[disp.group];  
+        out$sd$phi <- se.phis*object$params$phi;
+        names(out$sd$phi) <- colnames(object$y);
+        
         if(length(unique(disp.group))==p){
           names(out$sd$phi) <- colnames(object$y);
         }else if(!is.null(names(disp.group))){
@@ -418,7 +432,7 @@ se.gllvm <- function(object, ...){
   } else {
     #Without traits#
     pars <- objrFinal$par
-    if(family %in% c("ZIP","ZINB")) {
+    if(family %in% c("ZIP","ZINB","ZIB", "ZNIB")) {
       p0i <- names(pars)=="lg_phi"
       p0 <- pars[p0i]
       p0 <- p0+runif(p,0,0.001)
@@ -475,7 +489,7 @@ se.gllvm <- function(object, ...){
     if(quadratic == FALSE){incl[names(objrFinal$par)=="lambda2"]<-FALSE}
     if(familyn!=7 && familyn!=12) incl[names(objrFinal$par)=="zeta"] <- FALSE
     if(familyn==0 || familyn==2 || familyn==7 || familyn==8) incl[names(objrFinal$par)=="lg_phi"] <- FALSE
-    if(familyn!=11) incl[names(objrFinal$par)=="lg_phiZINB"] <- FALSE
+    if(!familyn %in% c(11, 14)) incl[names(objrFinal$par)=="lg_phiZINB"] <- FALSE
     
     if((num.lv+num.lv.c)>0){
       inclr[names(objrFinal$par)=="u"] <- TRUE;
@@ -622,8 +636,9 @@ se.gllvm <- function(object, ...){
       out$sd$LvXcoef <- se.LvXcoef
     }
     
-    if((num.lv.c+num.lv)>0){
+    if((num.lv.c+num.lv)>0){ # exp transformation
       se.sigma.lv <- se$sigmaLV;
+      # se.sigma.lv <- se$sigmaLV*object$params$sigma.lv;
     }
     
     if(num.lv > 0&(num.lv.c+num.RR)==0) {
@@ -691,15 +706,16 @@ se.gllvm <- function(object, ...){
     #   out$sd$se.thetaH <- se.thetaH
     # }
     
-    if(family %in% c("ZINB")) {
-      se.ZINB.lphis <- se$lg_phiZINB[disp.group];  out$sd$ZINB.inv.phi <- se.ZINB.lphis*object$params$ZINB.inv.phi;
+    if(family %in% c("ZINB", "ZNIB")) {
+      se.ZINB.lphis <- se$lg_phiZINB[disp.group];  
+      if(family == "ZINB")out$sd$ZINB.inv.phi <- se.ZINB.lphis*object$params$ZINB.inv.phi;
       out$sd$ZINB.phi <- se.ZINB.lphis*object$params$ZINB.phi;
       names(out$sd$ZINB.phi) <- colnames(object$y);
       
       if(!is.null(names(disp.group))){
         try(names(out$sd$ZINB.phi) <- names(disp.group),silent=T)
       }
-      names(out$sd$ZINB.inv.phi) <-  names(out$sd$ZINB.phi)
+      if(family == "ZINB")names(out$sd$ZINB.inv.phi) <-  names(out$sd$ZINB.phi)
     }
     
     if((num.lv+num.lv.c)>0){
@@ -743,9 +759,21 @@ se.gllvm <- function(object, ...){
       }
     }
     
-    if(family %in% c("ZIP","ZINB")) {
+    if(family %in% c("ZIP","ZINB","ZIB")) {
       se.phis <- se$lg_phi[disp.group];
       out$sd$phi <- se.phis*exp(p0)/(1+exp(p0))^2;#
+      if(length(unique(disp.group))==p){
+        names(out$sd$phi) <- colnames(object$y);
+      }else if(!is.null(names(disp.group))){
+        try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+      }else{
+        names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+      }
+    }else if(family == "ZNIB"){
+      se.phis <- se$lg_phi[disp.group];  
+      out$sd$phi <- se.phis*object$params$phi;
+      names(out$sd$phi) <- colnames(object$y);
+      
       if(length(unique(disp.group))==p){
         names(out$sd$phi) <- colnames(object$y);
       }else if(!is.null(names(disp.group))){
@@ -754,6 +782,7 @@ se.gllvm <- function(object, ...){
         names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
       }
     }
+    
     if(!isFALSE(object$randomB)&(num.lv.c+num.RR)>0){
       if(object$randomB!="iid"){
         if(object$randomB=="LV"){
